<?php

class WxPayConfig
{
	public function set($appid, $appsecret, $merchantid, $notify, $key, $ssl, $type, $timeout = 600, $ip = '')
	{
		$this->appid = $appid;
		$this->appsecret = $appsecret;
		$this->merchantid = $merchantid;
		$this->notify = $notify;
		$this->key = $key;
		$this->ssl = $ssl;
		$this->type = $type;
		$this->timeout = $timeout;
		$this->ip = $ip;
		return $this;
	}
	public function GetIp()
	{
		return $this->ip;
	}
	public function GetAppId()
	{
		return $this->appid;
	}
	public function GetMerchantId()
	{
		return $this->merchantid;
	}
	public function GetNotifyUrl()
	{
		return $this->notify;
	}
	public function GetSignType()
	{
		return "HMAC-SHA256";
	}
	public function GetProxy(&$proxyHost, &$proxyPort)
	{
		$proxyHost = "0.0.0.0";
		$proxyPort = 0;
	}
	public function GetReportLevenl()
	{
		return 1;
	}
	public function GetKey()
	{
		return $this->key;
	}
	public function GetAppSecret()
	{
		return $this->appsecret;
	}
	public function GetSSLCertPath(&$sslCertPath, &$sslKeyPath)
	{
		$sslCertPath = $this->ssl['cert'];
		$sslKeyPath = $this->ssl['key'];
	}
	public function GetTimeOut()
	{
		return $this->timeout;
	}
	public function GetType()
	{
		return $this->type;
	}
}
